const renderList = items =>
  items.length === 1
    ? items[0]
    : `${items.slice(0, -1).join(", ")} and ${items[items.length - 1]}`;

const FormLink = Vue.component("form-link", {
  props: ["linkNumber", "value", "interfaces", "ifaceMap"],
  template: `
    <div>
      <h3>Link {{ linkNumber }}</h3>
      <ev-select
        label="interface"
        :options="interfaces"
        :value="value"
        @input="onChange"
        :valid="getValidation"
      />
    </div>
  `,
  computed: {
    getValidation() {
      const { value, ifaceMap } = this;
      const ifaceUses = value in ifaceMap ? ifaceMap[value] : [];
      return ifaceUses.length < 2
        ? V.VALID
        : V.invalid("Interface used more than once: " + renderList(ifaceUses));
    }
  },
  methods: {
    onChange(event) {
      this.$emit("input", event);
    }
  }
});

/**
 * form-links
 */
Vue.component("form-links", {
  props: ["interfaces", "usedInterfaces"],
  data() {
    return {
      count: 2,
      links: []
    };
  },
  template: `
    <form @submit.prevent="onSubmit">
      <h2>Links</h2>
      <ev-select
        label="Link count"
        :value="count"
        :options="countOptions"
        @input="setLinkCount"
        :focus="true"
      />
      <form-link
        v-for="(link, index) in links"
        :key="link.linkNumber"
        :linkNumber="link.linkNumber"
        :value="link.iface"
        :interfaces="interfaces"
        :ifaceMap="ifaceMap"
        @input="links[index].iface = $event"
      />

      <input
        type="button"
        class="btn btn-primary"
        value="back"
        @click="$emit('back')"
      />
      <input
        class="btn btn-primary pull-right"
        type="submit"
        value="next"
        :disabled="!validState"
      />
    </form>
  `,

  /**
   * Life cycle hooks
   */
  beforeMount() {
    this.setLinkCount(this.count);
  },

  /**
   * Computed properties
   */
  computed: {
    // Options for the link count selector
    countOptions() {
      const usedInterfaceCount = Object.keys(this.usedInterfaces).length;
      const freeInterfaceCount = this.interfaces.length - usedInterfaceCount;

      let options = [];

      for (var index = 1; index <= freeInterfaceCount; index++) {
        options.push(index);
      }

      return options;
    },

    // Creates an Object with the interfaces as the keys and
    // an array of the place it's used as the value.
    // {
    //   eth0: ['WAN'],
    //   eth1: ['Link1']
    // }
    // Any interface with more than one 'use' indicates a clash.
    ifaceMap() {
      const iMap = {};
      const { links, usedInterfaces } = this;

      const assign = (interface, name) =>
        (iMap[interface] = (iMap[interface] || []).concat(name));

      Object.keys(usedInterfaces).forEach(iface =>
        assign(iface, usedInterfaces[iface])
      );

      links.forEach(link => assign(link.iface, `Link ${link.linkNumber}`));

      return iMap;
    },

    // Returns true in none of the interfaces are used for more
    // than one thing.
    validState() {
      const { ifaceMap } = this;

      for (let iface in ifaceMap) {
        if (ifaceMap[iface].length > 1) return false;
      }

      return true;
    }
  },

  /**
   * Methods
   */
  methods: {
    // Adds or removes link in the links array
    setLinkCount(value) {
      value = Number(value);

      if (value < this.count) {
        this.links = this.links.slice(0, value);
        this.count = value;
        return;
      }

      while (this.links.length < value) {
        this.links.push({
          linkNumber: this.links.length + 1,
          iface: this.nextFreeInterface()
        });
      }

      this.count = value;
    },

    // Return the first interface that is not in the
    // interface map.
    nextFreeInterface() {
      const usedInterfaces = Object.keys(this.ifaceMap);

      return this.interfaces.find(
        interface => !usedInterfaces.includes(interface)
      );
    },

    onSubmit() {
      this.$emit("update", [].concat(this.links));
    }
  }
});
